<?php

/**
 * 1997-2019 Quadra Informatique.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0) that is available
 * through the world-wide-web at this URL: http://www.opensource.org/licenses/OSL-3.0
 * If you are unable to obtain it through the world-wide-web, please send an email
 * to modules@quadra-informatique.fr so we can send you a copy immediately.
 *
 * @author    Quadra Informatique <modules@quadra-informatique.fr>
 * @copyright 1997-2019 Quadra Informatique
 * @license   http://www.opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 *
 * @category    Bnp
 */

namespace Bnpparibas\Axepta\Model;

/**
 * Class ConfigProvider.
 */
class AxeptaConfigProvider implements \Magento\Checkout\Model\ConfigProviderInterface
{
    public const CODE = \Bnpparibas\Axepta\Model\Method\Axepta::METHOD_CODE;

    protected $url;
    protected $helperData;
    protected $customer;
    protected $merchantAccount;
    protected $methodAllowedCountryFactory;
    protected $methodPositionFactory;
    protected $cart;
    protected $date;
    protected $assets;
    protected $request;
    protected $methodsApi;
    protected $scopeConfig;
    protected $isLogActive;
    /**
     * Logger.
     *
     * @var LoggerInterface
     */
    protected $loggerAxepta;

    public function __construct(
        \Magento\Backend\Model\Url $url,
        \Bnpparibas\Axepta\Helper\Data $helperData,
        \Magento\Customer\Model\Session $customer,
        MerchantAccount $merchantAccount,
        MethodAllowedCountryFactory $methodAllowedCountryFactory,
        PositionFactory $methodPositionFactory,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Checkout\Model\Cart $cart,
        \Magento\Framework\Stdlib\DateTime\DateTime $date,
        \Magento\Framework\View\Asset\Repository $assets,
        \Magento\Framework\App\RequestInterface $request,
        \Bnpparibas\Axepta\Logger\Logger $loggerAxepta
    ) {
        $this->url = $url;
        $this->helperData = $helperData;
        $this->customer = $customer;
        $this->merchantAccount = $merchantAccount;
        $this->methodAllowedCountryFactory = $methodAllowedCountryFactory;
        $this->methodPositionFactory = $methodPositionFactory;
        $this->cart = $cart;
        $this->date = $date;
        $this->assets = $assets;
        $this->request = $request;

        $this->scopeConfig = $scopeConfig;

        $this->isLogActive = $this->scopeConfig->getValue(
            \Bnpparibas\Axepta\Model\Api\Service::AXEPTA_LOG_ACTIVE,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
        $this->loggerAxepta = $loggerAxepta;
    }

    /**
     * Retrieve assoc array of checkout configuration.
     *
     * @return array
     */
    public function getConfig($isoCountry = '')
    {
        if (empty($isoCountry)) {
            return [];
        }

        $params = ['_secure' => $this->request->isSecure()];
        $allowedMerchantAccount = $this->helperData->getAllowedMerchantAccount($isoCountry);

        if ($allowedMerchantAccount === false) {
            return [];
        }

        $allowedMethods = $this->helperData->getAllowedMethods($allowedMerchantAccount);
        $grandTotal = $this->cart->getQuote()->getGrandTotal();

        if ($this->isLogActive) {
            $this->loggerAxepta->info('AXEPTA - GetConfig Payments');
            $this->loggerAxepta->info('Grand Total: '.print_r($grandTotal, true));
            $this->loggerAxepta->info('Code iso: '.print_r($isoCountry, true));
            $this->loggerAxepta->info('Payments: '.print_r($allowedMethods, true));
        }

        $allowedMethodsAfterCustomerIso = [];
        foreach ($allowedMethods as $method) {
            $checkCountry = $this->methodAllowedCountryFactory->create()->getCollection()
                    ->addFieldToFilter('method_id', ['eq' => $method['id']])
                    ->addFieldToFilter('country', ['in' => [$isoCountry, 'ALL']]);

            if (!empty($checkCountry->getData())) {
                // Check method specifique ici (comme cetelem)
                switch ($method['code']) {
                    case 'FC3':
                    case 'FC4':
                        if ($grandTotal >= 90 && $grandTotal <= 3000) {
                            $allowedMethodsAfterCustomerIso[] = $method;
                        }
                        break;
                    case 'PRE':
                        if ($grandTotal >= 150 && $grandTotal <= 16000) {
                            $allowedMethodsAfterCustomerIso[] = $method;
                        }
                        break;
                    default:
                        $allowedMethodsAfterCustomerIso[] = $method;
                        break;
                }
            }
        }

        $allowedMethodsAfterCustomerIso = $this->sortPaygates($isoCountry, $allowedMethodsAfterCustomerIso);

        foreach ($allowedMethodsAfterCustomerIso as &$method) {
            if ($isoCountry != 'FR' && $method['libelle'] == 'CB/VISA/Mastercard') {
                $method['libelle'] = 'VISA/Mastercard';
            }
            $listImages = explode('/', $method['libelle'] ?? '');
            foreach ($listImages as $img) {
                if ($isoCountry != 'FR' && $listImages == 'CB') {
                    continue;
                }
                $method['images'][] = $this->assets->getUrlWithParams('Bnpparibas_Axepta::images/'.preg_replace('/\s+/', '_', strtoupper($img)).'.png', $params);
            }
        }
        $userCardCollection = $this->helperData->getCustomerOneclickCardList();
        $userCards = $this->helperData->getDataFromCollection($userCardCollection);
        foreach ($userCards as &$card) {
            $card['pcnr'] = __('Card ending with').' •••-X'.substr($card['pcnr'], -3).'';
            $card['idPcnr'] = substr($card['pcnr'], -3);
            $card['image'] = $this->assets->getUrlWithParams('Bnpparibas_Axepta::images/'.preg_replace('/\s+/', '_', strtoupper($card['ccbrand'])).'.png', $params);
        }

        return [
            'axepta' => [
                'methods' => (count($allowedMethodsAfterCustomerIso) > 0) ? $allowedMethodsAfterCustomerIso : false,
                'recurringPayment' => [
                    'isActive' => $this->helperData->isRecurringPaymentIsAllow($allowedMerchantAccount),
                ],
                'oneClick' => [
                    'isActive' => (bool) $allowedMerchantAccount->getOneclickPaymentActive(),
                    'userCards' => $userCards,
                ],
                'is3dSecureActive' => $allowedMerchantAccount->get3dSecureActive(),
                'displayMethod' => $allowedMerchantAccount->getDisplayMethod(),
                'methodName' => $allowedMerchantAccount->getMethodName(),
            ],
        ];
    }

    private function sortPaygates($isoCountry, $paygates)
    {
        $countryPosition = ['FR', 'BE', 'DE', 'IT', 'NL', 'GB', 'ES', 'PT'];
        if (!in_array($isoCountry, $countryPosition)) {
            $isoCountry = 'def';
        }

        $paygatesPosition = $this->methodPositionFactory->create()->getCollection()
                ->addFieldToFilter('iso', ['eq' => $isoCountry]);

        $positions = [];
        $newPaygateOrder = [];

        foreach ($paygatesPosition as $paygatePosition) {
            $positions[$paygatePosition['trigram']] = $paygatePosition['position'];
        }

        foreach ($paygates as $paygate) {
            if (isset($positions[$paygate['code']])) {
                $newPaygateOrder[$positions[$paygate['code']]] = $paygate;
            }
        }

        ksort($newPaygateOrder);
        $newArray = [];
        foreach ($newPaygateOrder as $value) {
            $newArray[] = $value;
        }

        return $newArray;
    }
}
