<?php

/**
 * 1997-2016 Quadra Informatique
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0) that is available
 * through the world-wide-web at this URL: http://www.opensource.org/licenses/OSL-3.0
 * If you are unable to obtain it through the world-wide-web, please send an email
 * to modules@quadra-informatique.fr so we can send you a copy immediately.
 *
 * @author    Quadra Informatique <modules@quadra-informatique.fr>
 * @copyright 1997-2016 Quadra Informatique
 * @license   http://www.opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 *
 * @category    Bnp
 * @package     Bnpparibas_Axepta
 */

namespace Bnpparibas\Axepta\Model\Method;

class AxeptaRecurring extends Axepta
{

    const METHOD_CODE = 'axepta_recurring';
    const URL_CAPTURE = 'https://paymentpage.axepta.bnpparibas/capture.aspx';

    protected $_code = self::METHOD_CODE;

    /**
     * @return string payment action
     */
    public function getConfigPaymentAction()
    {
        return $this->getInfoInstance()->getAdditionalInformation('action');
    }

    public function authorize(\Magento\Payment\Model\InfoInterface $payment, $amount)
    {
        parent::authorize($payment, $amount);
        if ($this->getConfigPaymentAction() == self::ACTION_AUTHORIZE) {
            $payment->setIsTransactionPending(1);
        }
        return $this;
    }

    /**
     * Check refund availability
     *
     * @return bool
     */
    public function canRefund()
    {
        return true;
    }

    /**
     * Check partial refund availability for invoice
     *
     * @return bool
     */
    public function canRefundPartialPerInvoice()
    {
        return true;
    }

    public function capture(\Magento\Payment\Model\InfoInterface $payment, $amount)
    {
        $paymentInformations = $payment->getAdditionalInformation();

        if (array_key_exists('captureMode', $paymentInformations)) {
            if ($paymentInformations['captureMode'] === 'manual') {
                $orderId = $payment->getOrder()->getId();

                $transactions = $this->transactionFactory->create();
                $transaction = $transactions->getCollection()->addFieldToFilter('order_id', ['eq' => $orderId])->getFirstItem();

                $merchantAccounts = $this->merchantAccountFactory->create();
                $merchantAccount = $merchantAccounts->getCollection()->addFieldToFilter('merchant_id', ['eq' => $transaction->getMerchantId()])->getFirstItem();

                $data = [
                    'MerchantID' => $transaction->getMerchantId(),
                    'PayID' => $transaction->getPayId(),
                    'TransID' => $transaction->getTransactionReference(),
                    'MAC' => $this->dataBnp->ctHMAC(
                        $transaction->getMerchantId(),
                        $amount * 100,
                        $payment->getOrder()->getOrderCurrencyCode(),
                        $merchantAccount->getHmacKey(),
                        $transaction->getPayId(),
                        $transaction->getTransactionReference()
                    ),
                    'Amount' => $amount * 100,
                    'Currency' => $payment->getOrder()->getOrderCurrencyCode(),
                    'RefNr' => $transaction->getRefNr()
                ];
                
                $this->curl->post(self::URL_CAPTURE, $data);
                $response = $this->curl->getBody();

                $payment->setTransactionId($payment->getId());
                $payment->setLastTransId($payment->getId());
                $payment->setTransactionAdditionalInfo(
                    \Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS, (array)$transaction->getData()
                );
            }
        }
        parent::capture($payment, $amount);
    }

    /**
     * Assign data to info model instance
     *
     * @param array|\Magento\Framework\DataObject $data
     * @return $this
     * @throws \Magento\Framework\Exception\LocalizedException
     * @api
     */
    public function assignData(\Magento\Framework\DataObject $data, $recurring = true)
    {
        $isRecurringPayment = $data->getAdditionalData('isRecurringPayment');
        
        parent::assignData($data, $isRecurringPayment);
        
        // Security check
        if ($data['method'] != self::METHOD_CODE) {
            throw new \Exception(__('Axepta BNP Paribas Recurring Error %1', '1'));
        }
        
        $paymentInfos = $this->getInfoInstance();
        $additionalInformation = $paymentInfos->getAdditionalInformation('axeptaMethodData');

        if ($additionalInformation['operation'] !== 'recurring') {
            $credentialOnFile = [
                'type' => ['unscheduled' => 'CIT'],
                'initialPayment' => true,
                'useCase' => 'ucof'
            ];
    
            $paymentInfos->setAdditionalInformation('credentialOnFile', self::encodeParamJson($credentialOnFile));
    
            $threeDSPolicy = [
                'challengePreference' => 'mandateChallenge'
            ];
    
            $paymentInfos->setAdditionalInformation('threeDSPolicy', self::encodeParamJson($threeDSPolicy));
        }

        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $customerSession = $objectManager->get('Magento\Customer\Model\Session');
        if (!$customerSession->isLoggedIn() && $additionalInformation['operation'] !== 'recurring') {
            throw new \Magento\Framework\Validator\Exception(__('You must be identified to pay with the Axepta subscription, you will be redirect to the login page'), null, [], 401);
        }

        
        return $this;
    }

         
    /**
     * Check whether payment method can be used
     *
     * @param \Magento\Quote\Api\Data\CartInterface|null $quote
     * @return bool
     * @deprecated 100.2.0
     */
    public function isAvailable(\Magento\Quote\Api\Data\CartInterface $quote = null)
    {
        $products = $quote->getAllVisibleItems();
        foreach ($products as $product) {
            if ($this->helper->isReccuringProduct($product)) {
                return true;
            }
        }
        return false;
    }

    /**
     * Encode param in json format
     * @param array $param
     * @return string
     */
    public static function encodeParamJson($param)
    {
        if (is_array($param)) {
            return base64_encode(json_encode($param));
        }
        return false;
    }
}
